/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 2003 - 2017  SEGGER Microcontroller GmbH & Co. KG        *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
----------------------------------------------------------------------
File        : PIDConf.c
Purpose     : Touch screen controller configuration
---------------------------END-OF-HEADER------------------------------
*/

#include "GUI.h"
#include "PIDConf.h"
#include "RTOS.h"
#include "stm32f429i_discovery_ts.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static U8 _IsInitialized;
static int _LayerIndex;

static OS_STACKPTR int Stack_Touch[128];
static OS_TASK TCB_TOUCH;

static void PID_X_Exec(void);
static void TouchTask(void);

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       TouchTask
*/
static void TouchTask(void) {
  while(1) {
    PID_X_Exec();
    OS_Delay(25);
  }
}

/*********************************************************************
*
*       PID_X_Exec
*/
static void PID_X_Exec(void) {
  TS_StateTypeDef TS_State;
  static GUI_PID_STATE StatePID;
  static int IsTouched;
  int        xSize;
  int        ySize;
  int        Swap;
  int        MirrorX;
  int        MirrorY;

  Swap    = LCD_GetSwapXYEx (_LayerIndex);
  MirrorX = LCD_GetMirrorXEx(_LayerIndex);
  MirrorY = LCD_GetMirrorYEx(_LayerIndex);
  
  if (_IsInitialized) {
    BSP_TS_GetState(&TS_State);
    StatePID.Layer = _LayerIndex;
    if (TS_State.TouchDetected) {
      IsTouched = 1;
      StatePID.Pressed = 1;
      if (Swap) {
        ySize = LCD_GetXSize();
        xSize = LCD_GetYSize();
        StatePID.y = (MirrorX) ? xSize - (int)(TS_State.X) : (int)(TS_State.X);
        StatePID.x = (MirrorY) ? ySize - (int)(TS_State.Y) : (int)(TS_State.Y);
      } else {
        xSize = LCD_GetXSize();
        ySize = LCD_GetYSize();
        StatePID.x = (MirrorX) ? xSize - (int)(TS_State.X) : (int)(TS_State.X);
        StatePID.y = (MirrorY) ? ySize - (int)(TS_State.Y) : (int)(TS_State.Y);
      }
      GUI_TOUCH_StoreStateEx(&StatePID);
    } else {
      if (IsTouched == 1) {
        IsTouched = 0;
        StatePID.Pressed = 0;
        GUI_TOUCH_StoreStateEx(&StatePID);
      }
    }
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       PID_X_SetLayerIndex
*/
void PID_X_SetLayerIndex(int LayerIndex) {
  _LayerIndex = LayerIndex;
}

/*********************************************************************
*
*       PID_X_Init
*/
void PID_X_Init(void) {
  int xSize, ySize;

  if (_IsInitialized == 0) {
    xSize = LCD_GetXSize();
    ySize = LCD_GetYSize();
    BSP_TS_Init(xSize, ySize);
    OS_CREATETASK(&TCB_TOUCH, "TouchTask", TouchTask, 101, Stack_Touch);
    _IsInitialized = 1;
  }
}

/*************************** End of file ****************************/
